#!/usr/bin/python3

"""A script to fill a protocol from Digital Eksamen.

How to use:

In Digital Eksamen at the protocol page for the exam, click "Export
Students" under the menu item of that name. This produces a protocol
csv-file with one line per student (plus a few lines more).

Inspect the csv-file and adjust the "Set specs..." section (below) in
the script appropriately.

Prepare a .txt data file with the grades, one line per student. The
first word of a line should be the student ID, the second word the
grade (more words are OK). Specifically, one of the grade listings
appearing in the output of the script getGrades.py can be used as the
file contents. Note: different protocol setups can use different
student IDs (exam numbers, student email addresses).

Then run as follows (the last named file will be created):

  makeProtokols.py grades.txt inputProtocol.csv outputProtocol.csv

The output is another protocol csv-file, which can then be imported in
Digital Eksamen at the protocol page for the exam (the import button
appears under the "Export Students" menu item).

IDs in the grades.txt file, which are not in the protocol file, are
listed on standard output when makeProtokols.py is run.

Note: If the Digital Eksamen interface is set to English when
exporting the protocol, it must also be set to English when importing
the protocol. The same applies to setting Danish.

"""

########## Set specs for the procotol in question (these can vary
########## between protocols):
# what column in the protocol contains the ID?
protocolStudentIDColumn = 0 # zero indexed

# what column in the protocol contains the grade
protocolGradeColumn = 1 # zero indexed
########## End of specs

########## Other procotol-dependent stuff which seems less variable:
# what is the delimiter in the csv-file for the protocol
protocolDelimiter = ";"

# what identifies the lines in the protocol as NOT being a data line,
# based on the first entry/word of the line:
def isNotGradeCVSLine(firstEntry):
    return ((firstEntry == '"Student number"'
             or firstEntry == '"Studienummer"'
             or firstEntry == '"Examnumber"'
             or firstEntry == "Eksamensnummer") # skip first heading
                                                # cvs line
            or (not firstEntry) # skip empty lines, i.e., second to
                                # last cvs line
            or (firstEntry == '"Possible grades"'
                or firstEntry == '"Mulige karakterer"')) # skip last
                                                         # explanatory cvs
                                                         # line

# Protocols exported from Digital Eksamen are latin-1 encoded (sigh,
# thought that we were all utf-8 these days)
protocolEncoding = "latin-1"
########## End of other procotol-dependent stuff

import sys

inputgradesfile = sys.argv[1]
inputprotokol = sys.argv[2]
outputprotokol = sys.argv[3]

with open(inputprotokol, 'r', encoding = protocolEncoding) as f:
    protocolrows = [l.split(protocolDelimiter) for l in f.readlines()]
with open(inputgradesfile, 'r') as f:
    datalines = [l.split() for l in f.readlines()]
    
# Make DB of grades:
data = {}
for l in datalines:
    studentID = l[0]
    grade = l[1]
    data[studentID] = grade

def getGrade(id):
    return data.get(id,"U") # if no grade recorded, return "U"

# make output cvs-file:
with open(outputprotokol, 'w', encoding = protocolEncoding) as fout:
    for r in protocolrows:
        # we need to skip (or rather, repeat) irrelevant rows of protocol
        if not isNotGradeCVSLine(r[0]):
            ID = r[protocolStudentIDColumn][1:-1] # remove "'s at end of strings
            potentialLineEnding = r[protocolGradeColumn] # if last column
            # put in grade in row:
            r[protocolGradeColumn] = f'"{getGrade(ID)}"' + potentialLineEnding
            if ID in data:
                del data[ID] # remove from grade db
        fout.write(protocolDelimiter.join(r)) # write (updated) row again

# Any input data left which did not go to the protocol?
if data:
    print()
    print("In data file, but not in protocol:")
    for ID in data.keys():
        print(ID, data[ID])
